/* agip.c -- An (almost) generic installation program
   Time-stamp: "97/02/02 01:26:07 mik"

   Copyright (C) 1996, 97 Christian Schenk <cschenk@berlin.snafu.de>

   This file is part of MiKTeX.

   MiKTeX is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
   
   MiKTeX is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with MiKTeX; if not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdarg.h>
#include <conio.h>
#include <string.h>
#include <ctype.h>
#include <process.h>

#include "../lib/miktex.version"
#define PROGRAM_NAME (VER_PRODUCTNAME_STR " " VER_PRODUCTVERSION_STR)
#define DEF_DEST_DIR "c:\\texmf"

#if defined (BETA)
#define HOWTOGET ""
#else
#define HOWTOGET "\
You can download missing files from this ftp location:\n\
\n\
   ftp://ftp.dante.de/tex-archive/systems/win32/miktex"
#endif

#define default_mode required
#define required 1
#define optional 2
#define ask 4
#define dontcare 8
#define ignore 16

static char destdir[_MAX_PATH];
static char destdirsansbackslash[_MAX_PATH];

static int
yesno (const char *question,
       ...)

{
  int ch;
  va_list marker;

  va_start (marker, question);
  do
    {
      char answer[100];
      vfprintf (stdout, question, marker);
      fflush (stdout);
      gets (answer);
      if (_strcmpi (answer, "yes") == 0)
	{
	  va_end (marker);
	  return (1);
	}
      if (_strcmpi (answer, "no") == 0)
	{
	  va_end (marker);
	  return (0);
	}
    }
  while (1);
}

static struct fileinfo
{
  const char *filename;
  int mode;
  const char *description;
}

filelist[] =

{
#include "install.dat"
};

static void
log (const char *msg,
     ...)

{
  va_list marker;

  va_start (marker, msg);
  vfprintf (stdout, msg, marker);
  va_end (marker);
  fputc ('\n', stdout);
}

static void
error (const char *msg,
       ...)

{
  va_list marker;

  fprintf (stderr, "Error: ");
  va_start (marker, msg);
  vfprintf (stderr, msg, marker);
  va_end (marker);
  fputc ('\n', stderr);
  
  exit (-1);
}

static void
usage (int rc)

{
  fprintf (stderr, "Usage: install\n");
  exit (rc);
}

static void
check_unzip_utility ()

{
  if (access (".\\unzip.exe", 0) != 0)
    error ("`unzip.exe' not found in current directory");
}

static void
make_path (const char *path)

/* borrowed from GNU File Utilities! */

{
  char *dirpath;
  struct stat statbuf;
  int retval = 0;

  dirpath = (char *) _alloca (strlen (path) + 1);
  strcpy (dirpath, path);

  if (stat (dirpath, &statbuf))
    {
      char *slash;
      char *cp;
      struct ptr_list
      {
	char *dirname_end;
	struct ptr_list *next;
      };
      struct ptr_list *p, *leading_dirs = NULL;

      slash = dirpath;
      if (slash[0] && slash[1] == ':')
	slash += 2;
      while (*slash == '/' || *slash == '\\')
	slash++;
      while ((cp = strchr (slash, '/')) || (cp = strchr (slash, '\\')))
	{
	  slash = cp;
	  *slash = '\0';
	  if (stat (dirpath, &statbuf))
	    {
	      if (mkdir (dirpath))
		{
		  perror (dirpath);
		  error ("cannot make directory `%s'", dirpath);
		}
	    }
	  else if (! (statbuf.st_mode & S_IFDIR))
	    error ("`%s' exists but is not a directory", dirpath);

	  *slash++ = '\\';

	  while (*slash == '/' || *slash == '\\')
	    slash++;
	}

      if (mkdir (dirpath))
	{
	  perror (dirpath);
	  error ("cannot make directory `%s'", dirpath);
	}
    }
  else if (! (statbuf.st_mode & S_IFDIR))
    error ("`%s' exists but is not a directory", dirpath);
}

static void
check_dest_dir (void)

{
  make_path (destdir);
}

static void
uncompress (size_t idx)

{
  int pargc;
  int rc;
  char *pargv[10];
  pargc = 0;  
  pargv[pargc++] = "unzip.exe";
  pargv[pargc++] = "-qq";
  pargv[pargc++] = "-o";
  pargv[pargc++] = "-d";
  pargv[pargc++] = destdir;
  pargv[pargc++] = filelist[idx].filename;
  pargv[pargc++] = 0;
  log ("Installing %s...", filelist[idx].description);
#if 1
  rc = _spawnv (_P_WAIT, ".\\unzip.exe", pargv);
  if (rc < 0)
    perror (".\\unzip.exe");
  if (rc != 0)
    error ("could not unpack archive `%s'", filelist[idx].filename);
#endif
}

static void
process_filelist (void)

{
#define nfiles (sizeof (filelist) / sizeof (struct fileinfo))

  size_t i;
  int missing = 0;
  int exitflag = 0;
  int first_question = 1;

  for (i = 0; i < nfiles; i++)
    {
      if (_access (filelist[i].filename, 0) != 0)
	{
	  if ((filelist[i].mode & required))
	    {
	      log ("Error: archive `%s' does not exist",
		   filelist[i].filename);
	      exitflag++;
	    }
	  else if ((filelist[i].mode & optional))
	    {
	      log ("Warning: archive `%s' does not exist",
		   filelist[i].filename);
	      missing++;
	    }
	}
      else if ((filelist[i].mode & ask) && ! exitflag)
	{
	  if (first_question)
	    {
	      log ("\n\nSelect optional components that you wish to install:\n");
	      first_question = 0;
	    }
	  if (! yesno ("\n%s ? [yes/no] ", filelist[i].description))
	    filelist[i].mode |= ignore;	  
	}
    }

  log ("\n\n");

  if (exitflag)
    error ("%d archive%s %s missing\n\n%s",
	   exitflag, exitflag > 1 ? "s" : "",
	   exitflag > 1 ? "are" : "is",
	   HOWTOGET);

  if (missing)
    {
      log ("%d archive%s %s missing",
	   missing, missing > 1 ? "s" : "",
	   exitflag > 1 ? "are" : "is");
      if (yesno ("\nDo you want to cancel the installation [yes/no]? "))
	{
	  log ("  %s", HOWTOGET);
	  exit (0);
	}
    }

  check_dest_dir ();

  log ("\n");
  for (i = 0; i < nfiles; i++)
    {
      if (_access (filelist[i].filename, 0) != 0)
	{
	  if (filelist[i].mode & required)
	    error ("required archive `%s' (%s) does not exist",
		   filelist[i].filename, filelist[i].description);
	}
      else if (! (filelist[i].mode & ignore))
	uncompress (i);
    }
}

#define CONFIGURE_EXE "configure.exe"

static void
configure_and_check ()

{
  FILE *stream;
  char command_line[300];
  char system_temp_dir[_MAX_PATH];
  char cwd[_MAX_PATH];
  int drive;
  int temp_drive;
  int pargc;
  char *pargv[10];
  int rc;
#if defined (_WIN32)
  char *oldpath = getenv ("PATH");
  char *newpath;
#endif

#if defined (_WIN32)
  if (oldpath == 0)
    error ("environment variable PATH is not defined");
  newpath = (char *) _alloca (strlen (oldpath)
			      + strlen ("PATH=")
			      + strlen (destdirsansbackslash)
			      + strlen ("\\miktex\\bin") + 10);
  sprintf (newpath, "PATH=%s\\miktex\\bin;%s",
	   destdirsansbackslash, oldpath);
  if (putenv (newpath) != 0)
    error ("could not modify PATH");
#endif

  sprintf (command_line, "%s\\miktex\\config\\%s",
	   destdirsansbackslash, CONFIGURE_EXE);

  log ("Writing filename database %s\\miktex\\config\\texmf.fndb...\n",
       destdirsansbackslash);

  pargc = 0;
  pargv[pargc++] = command_line;
  pargv[pargc++] = "-r";
  pargv[pargc++] = destdir;
  pargv[pargc++] = "-u";
  pargv[pargc++] = 0;
  
  rc = _spawnv (_P_WAIT, command_line, pargv);
  if (rc < 0)
    perror (command_line);
  if (rc != 0)
    error ("Configuration Utility failed");
  GetTempPath (sizeof (system_temp_dir), system_temp_dir);
  drive = _getdrive ();
  temp_drive = tolower (system_temp_dir[0]) - 'a' + 1;
  if (getcwd (cwd, sizeof (cwd)) == 0)
    {
      perror ("");
      error ("cannot determine current directory");
    }
  if (_chdrive (temp_drive) != 0)
    {
      perror (system_temp_dir);
      error ("cannot chdrive");
    }
  if (_chdir (system_temp_dir) != 0)
    {
      perror (system_temp_dir);
      error ("cannont chdir");
    }
  if ((stream = fopen ("test.tex", "w")))
    {
      size_t i;
      fprintf (stream, "\\hsize=10cm\\parindent=0pt\\raggedright%%\n\
Installation of MiK\\TeX\\ %s is complete.\n\n\
Do not forget to append {\\tt ", VER_PRODUCTVERSION_STR);
      for (i = 0; i < strlen (destdirsansbackslash); i++)
	{
	  if (destdirsansbackslash[i] == '\\')
	    fprintf (stream, "$\\backslash$");
	  else
	    fputc (destdirsansbackslash[i], stream);
	}
      fprintf (stream,"$\\backslash$miktex$\\backslash$bin} to the environment \
variable {\\tt PATH}.\n\nEnjoy!\n\
\\bye",
	       destdirsansbackslash);
      fclose (stream);
      log ("\n\nChecking that the installation worked:\n\n");
      log ("  1. Invoking TeX...\n");
      sprintf (command_line, "%s\\miktex\\bin\\tex.exe",
	       destdirsansbackslash);
      pargc = 0;
      pargv[pargc++] = command_line;
      pargv[pargc++] = "test";
      pargv[pargc++] = 0;
      rc = _spawnv (_P_WAIT, command_line, pargv);
      if (rc < 0)
	perror (command_line);
      if (rc)
	error ("Invocation of TeX failed");
      log ("\n  2. Opening DVI file...\n");
      sprintf (command_line, "%s\\miktex\\bin\\yap.exe",
	       destdirsansbackslash);
      pargc = 0;
      pargv[pargc++] = command_line;
      pargv[pargc++] = "test.dvi";
      pargv[pargc++] = 0;
      rc = _spawnv (_P_WAIT, command_line, pargv);
      if (rc < 0)
	perror (command_line);
      if (rc)
	error ("Invocation of YAP failed");
    }
  if (_chdrive (drive) != 0)
    perror (cwd);
  if (chdir (cwd) != 0)
    perror (cwd);
}

static void
welcome (void)

{
  log ("Welcome to %s Installation Utility. This program will install\n\
%s on your system.\n\n", PROGRAM_NAME, PROGRAM_NAME);
}

static char *
find_dest_dir (char *dest_dir,
	       const char *def_dest_dir)

{
  HKEY hKey;
  LONG res;
  res = RegOpenKeyEx (HKEY_LOCAL_MACHINE,
		      "Software\\MiK\\MiKTeX\\CurrentVersion\\MiKTeX",
		      0,
		      KEY_READ,
		      &hKey);
  if (res != ERROR_SUCCESS)
    res = RegOpenKeyEx (HKEY_LOCAL_MACHINE,
			"Software\\MiK\\MiKTeX\\MiKTeX",
			0,
			KEY_READ,
			&hKey);
  if (res == ERROR_SUCCESS)
    {
      DWORD type;
      BYTE value[2048];
      DWORD len = sizeof(value);
      res = RegQueryValueEx (hKey,
			     "Root Directory",
			     0,
			     &type,
			     value,
			     &len);
      RegCloseKey (hKey);
      if (res == ERROR_SUCCESS)
	strcpy (dest_dir, value);
      else
	strcpy (dest_dir, def_dest_dir);
    }
  else
    strcpy (dest_dir, def_dest_dir);
  return (0);
}

static void
get_dest_dir (void)

{
  size_t i;
  size_t len;
  char def_dest_dir[300];
  find_dest_dir (def_dest_dir, DEF_DEST_DIR);
  log ("%s will be installed in `%s'.\n", PROGRAM_NAME, def_dest_dir);
  log ("To install to this directory, press <RETURN>.\n");
  log ("To install to a different directory, enter the name of another \
directory.\n");
  log ("You can choose not to install %s. Enter `cancel' to cancel the\n\
installation.\n", PROGRAM_NAME);
  printf ("Destination Directory: ");
  gets (destdir);
  if (*destdir == 0)
    strcpy (destdir, def_dest_dir);
  if (strcmp (destdir, "cancel") == 0)
    exit (0);

  len = strlen (destdir);
  for (i = 0; i < len; i++)
    {
      if (destdir[i] == '/')
	destdir[i] = '\\';
    }
  strcpy (destdirsansbackslash, destdir);
  if (destdirsansbackslash[len - 1] == '\\')
    destdirsansbackslash[len - 1] = 0;
  if (! isalpha (destdir[0]) || destdir[1] != ':' || destdir[2] != '\\')
    error ("you must enter an absolute pathname (e.g. c:\\texmf)");
}

int
main (int argc,
      char **argv)

{
  if (argc != 1)
    usage (-1);
  welcome ();
  get_dest_dir ();
  check_unzip_utility ();
  process_filelist ();
  configure_and_check ();
  log ("\n\nInstallation is complete. Please append `%s\\miktex\\bin' to\n\
the environment variable `PATH' and reboot the system.",
       destdirsansbackslash);
  return (0);
}
